#Simulation for table 2, correlated predictor versus true predictor
#progression-regression

#### Clear Workspace + Load Packages
rm(list=ls(all=TRUE))
library(survival)
library("mvtnorm")
source("MSC_C.R")

set.seed(1234)

#### number of sims to use
nsim <- 1000
#### number of observations to use
n <- 150
num <- n*nsim

#### set covariate
#true parameter values
beta01 <- 3.5
beta02 <- .5
beta03 <- 1
beta1 <- -1 #lower is riskier
beta2 <- 2
beta3 <- -1
tau <- 1


#step 1: draw covariate to use for each simulation
sigma <- matrix(c( 3,2.7,
                  2.7, 3), ncol=2)
x <- rmvnorm(num,sigma=sigma)
frailty1 <- rnorm(num)
frailty2 <- rnorm(num)
frailty3 <- rnorm(num)
z1 <- beta1*x[,1]
z2 <- beta2*x[,1]
z3 <- beta2*x[,1]
lp1 <- exp(beta01+z1+frailty1)
lp2 <- exp(beta02+z2-frailty1)
lp3 <- exp(beta03+z3)
lp4 <- exp(beta01+z1+frailty2)
lp5 <- exp(beta02+z2-frailty2)
lp7 <- exp(beta01+z1+frailty3)
lp8 <- exp(beta02+z2-frailty3)

t_tilde1 <- rweibull(num,1/tau,lp1) #true t_i for all simulated datasets
t_tilde2 <- rweibull(num,1/tau,lp2) #true t_i for all simulated datasets
t_tilde3 <- rweibull(num,2,lp3)
t_tilde4 <- rweibull(num,1/tau,lp4) #true t_i for all simulated datasets
t_tilde5 <- rweibull(num,1/tau,lp5) #true t_i for all simulated datasets
t_tilde6 <- rweibull(num,2,lp3)
t_tilde7 <- rweibull(num,1/tau,lp7) #true t_i for all simulated datasets
t_tilde8 <- rweibull(num,1/tau,lp8) #true t_i for all simulated datasets
t_tilde9 <- rweibull(num,2,lp3)

t_tilde10 <- ifelse(t_tilde1<=t_tilde2,t_tilde1,
                  ifelse(t_tilde4<=t_tilde5,t_tilde2+t_tilde3+t_tilde4,
                         ifelse(t_tilde7<=t_tilde8,t_tilde2+t_tilde3+t_tilde5+t_tilde6+t_tilde7,31)))
t_tilde <- ifelse(t_tilde1<=t_tilde2,t_tilde1,
                  ifelse(t_tilde4<=t_tilde5,t_tilde1+t_tilde3+t_tilde4,
                         ifelse(t_tilde7<=t_tilde8,t_tilde1+t_tilde3+t_tilde5+t_tilde6+t_tilde7,
                         t_tilde1+t_tilde3+t_tilde5+t_tilde6+t_tilde8+t_tilde9)))
                         
#### set censoring
cens <- pmin(rexp(nsim*n,.05),runif(nsim*n,24,30))  #random censoring

table(cens[t_tilde2<t_tilde1 & t_tilde5<t_tilde4 & t_tilde8<t_tilde7]>t_tilde[t_tilde2<t_tilde1 & t_tilde5<t_tilde4 & t_tilde8<t_tilde7])
summary(t_tilde[t_tilde2<t_tilde1 & t_tilde5<t_tilde4 & t_tilde8<t_tilde7 & cens>t_tilde])

#cens <- rep(13.5,num)  #type I censoring
t1 <- pmin(t_tilde10,cens)
t2 <- pmin(t_tilde2,cens,t_tilde10)
d1 <- I(t_tilde10<=cens)
d2 <- I(t_tilde2<=pmin(cens,t_tilde10))
xsam_all <- data.frame(t1,d1,t2,d2,x)

#### percent of event 1 observed
table(d1)/length(d1)

#### percent of event 2 observed
table(d2)/length(d2)

#### percent of event 1 observed
table(d1,d2)/length(d1)

d1c <- 1-d1
kmm1 <- survfit(Surv(t1,d1c)~ 1)
tau1 <- max(kmm1$time[kmm1$surv>=.25])

simres <- data.frame()
for (simno in 1:nsim){
   #### restrict data to the simulation
   print(simno)
   lower <- (simno-1)*n+1
   upper <- simno*n
   xsam <- xsam_all[lower:upper,]

   ####fit phreg model
   xsam$z11 <- xsam$X1
   xsam$z12 <- -1*xsam$X1
   xsam$z21 <- xsam$X2
   xsam$z22 <- -1*xsam$X2

   simres <- rbind(simres,MSC_C(1,.75,xsam$d1,xsam$t1,xsam$d2,xsam$t2,xsam$z11,xsam$z12,xsam$z21,xsam$z22,tau1))
}

save(simres,file="Cp_res2.RData")
