#Simulation for table 2, true predictor versus true predictor+correlated predictor
#illness-death

#### Clear Workspace + Load Packages
rm(list=ls(all=TRUE))
library(survival)
library("mvtnorm")
source("MSC_C_3.R")

set.seed(1234)

#### number of sims to use
nsim <- 1000
#### number of observations to use
n <- 150
num <- n*nsim

#### set covariate
#true parameter values
beta01 <- 2.9
beta02 <- 3.7
beta1 <- -1 #lower is riskier
beta2 <- -.5
tau <- 1


#step 1: draw covariate to use for each simulation
sigma <- matrix(c( 3,.5,
                  .5, 3), ncol=2)
x <- rmvnorm(num,sigma=sigma)
x4 <- rnorm(num)  #predictive of only death

z1 <- beta1*(x[,1])
z2 <- beta2*(x[,1]+x4)
lp1 <- exp(beta01+z1)
lp2 <- exp(beta02+z2)

t_tilde2 <- rweibull(num,1/tau,lp1) #time from healthy to illness
t_tilde1 <- rweibull(num,1/tau,lp2) #time from illness to death

#time to death
t_tilde10 <- t_tilde1+t_tilde2

#### set censoring
cens <- pmin(rexp(nsim*n,.05),runif(nsim*n,24,30))  #random censoring

t1 <- pmin(t_tilde10,cens)  #time to death
t2 <- pmin(t_tilde2,cens)  #time to illness
d1 <- I(t_tilde10<=cens)  #indicator variable for death
d2 <- I(t_tilde2<=cens)  #indicator variable for illness
xsam_all <- data.frame(t1,d1,t2,d2,x,x3,x4,t_tilde10,t_tilde2)

#### percent of event 1 observed
table(d1)/length(d1)

#### percent of event 2 observed
table(d2)/length(d2)

d1c <- 1-d1
kmm1 <- survfit(Surv(t1,d1c)~ 1)
tau1 <- max(kmm1$time[kmm1$surv>=.25])

simres <- data.frame()
for (simno in 1:nsim){
   #### restrict data to the simulation
   print(simno)
   lower <- (simno-1)*n+1
   upper <- simno*n
   xsam <- xsam_all[lower:upper,]

   ####fit phreg model
   coxmod1 <- coxph(Surv(t1,d1) ~ X1 + X2,data=xsam)   ### fit cox model 1
   harrellc21 <- summary(coxmod1)$concordance[1]   ### concordance   
   coxmod2 <- coxph(Surv(t2,d2) ~ X1 + X2,data=xsam)   ### fit cox model 2
   harrellc22 <- summary(coxmod2)$concordance[1]   ### concordance   
  
   xsam$z11 <- xsam$X1
   xsam$z12 <- xsam$X1
   xsam$z21 <- coxmod1$coeff[1]*xsam$X1+coxmod1$coeff[2]*xsam$X2  ### save linear predictor of cox model
   xsam$z22 <- coxmod2$coeff[1]*xsam$X1+coxmod2$coeff[2]*xsam$X2  ### save linear predictor of cox model

   simres <- rbind(simres,MSC_C(1,.75,xsam$d1,xsam$t1,xsam$d2,xsam$t2,xsam$z11,xsam$z12,xsam$z21,xsam$z22,tau1))
}

save(simres,file="Cp_res3.RData")
