#Table 1: false discovery rate simulations
#progression-regression
#biomarker for progression only

#### Clear Workspace + Load Packages
rm(list=ls(all=TRUE))
library(survival)
library("mvtnorm")
set.seed(1234)
dirname <- "/home/cheunglc/cindex2"

#### number of sims to use
nsim <- 1000
#### number of observations to use
n <- 150
num <- n*nsim

#### set covariate
#true parameter values
beta01 <- 2.9
beta02 <- 3.7
beta1 <- -1 #lower is riskier
beta2 <- -.5
tau <- 1


#step 1: draw covariate to use for each simulation
sigma <- matrix(c( 3,.5,.5,.5,.5,.5, 0, 0, 0, 0, 0,
                  .5, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                  .5, 0, 3, 0, 0, 0, 0, 0, 0, 0, 0,
                  .5, 0, 0, 3, 0, 0, 0, 0, 0, 0, 0,
                  .5, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0,
                  .5, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0,
                   0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0,
                   0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0,
                   0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0,
                   0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0,
                   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3), ncol=11)
x <- rmvnorm(num,sigma=sigma)
z1 <- beta1*(x[,1])
z2 <- beta2*(x[,1])
lp1 <- exp(beta01+z1)
lp2 <- exp(beta02+z2)

t_tilde2 <- rweibull(num,1/tau,lp1) #time from healthy to illness
t_tilde1 <- rweibull(num,1/tau,lp2) #time from illness to death

#time to death
t_tilde10 <- t_tilde1+t_tilde2
                         
#### set censoring
cens <- pmin(rexp(nsim*n,.05),runif(nsim*n,24,30))  #random censoring

#cens <- rep(13.5,num)  #type I censoring
t1 <- pmin(t_tilde10,cens)
t2 <- pmin(t_tilde2,cens,t_tilde10)
d1 <- I(t_tilde10<=cens)
d2 <- I(t_tilde2<=cens)  #indicator variable for illness
xsam_all <- data.frame(t1,d1,t2,d2,x)

#### percent of event 1 observed
table(d1)/length(d1)

#### percent of event 2 observed
table(d2)/length(d2)

d1c <- 1-d1
kmm1 <- survfit(Surv(t1,d1c)~ 1)
tau1 <- max(kmm1$time[kmm1$surv>=.25])

for (k in 101:150){
  #### restrict data to the simulation
  print(k)
  lower <- (k-101)*n*20+1
  upper <- (k-100)*n*20
  xsam_all <- xsam_all[lower:upper,]
  save(xsam_all,file=paste0("y",k,".RData"))

  #create R file
  write(paste0("load('y",k,".RData')"),paste0(dirname,"/fdr_",k,".R"))
  file.append(paste0(dirname,"/fdr_",k,".R"),"fdr2.R")
  com <- paste0("save(simres,file='",dirname,"/wl_10var",k,".RData')")
  cat(com,file=paste0(dirname,"/fdr_",k,".R"), append=TRUE)  
  
  #create swarm file
  command <- paste0("R --vanilla <",dirname,"/fdr_",k,".R >",dirname,"/lyg_",k,".out")
  write(command,paste0(dirname,"/swarm_fdr",k))
    
  #submit swarm file
  com<-paste0("swarm -g 1 -f ",dirname,"/swarm_fdr",k," --module R", " --time 3-00:00:00")
  system(com)
}
