library(survival)
setwd("C:/Users/Li/Documents/concordance")
load("dpp.RData")
source("MSC_C_3.R")

#subset to blacks
dpp <- subset(dpp,RACE6==2)

table(dpp$DIABF)

#   0    1 
#2552  682 

table(dpp$NGR_EVENT)

#   0    1 
#2316  918 

summary(dpp$agerand)
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#  25.45   42.96   49.83   50.61   57.67   85.24 

sd(dpp$agerand)
#[1] 10.68364

table(dpp$tx,useNA="always")
#1079 lifestyle, #1073 drugs, #1082 placebo
dpp$lifestyle_tx <- ifelse(dpp$tx==1,1,0)
dpp$drug_tx <- ifelse(dpp$tx==2,1,0)

table(dpp$sex,useNA="always")
#1043 M, 2191 F
dpp$female <- ifelse(dpp$sex == 2,1,0)

table(dpp$RACE6)
#   1    2    3    4    5 
#1768  645  508  142  171 
dpp$black <- ifelse(dpp$RACE6==2,1,0)
dpp$hisp <- ifelse(dpp$RACE6==3,1,0)
dpp$asian <- ifelse(dpp$RACE6==4,1,0)
dpp$natamer <- ifelse(dpp$RACE6==5,1,0)

table(dpp$famhxdb,useNA="always")
#  0    1 <NA> 
# 989 2243    2 

table(dpp$HYPERTENSION_HIST,useNA="always")
#   0    1 <NA> 
# 583 2468  183 

table(dpp$HRT_HIST,useNA="always")
#   0    1 <NA> 
# 2100 1134    0 

table(dpp$DYSLIPID_HIST,useNA="always")
#   0    1 <NA> 
# 849 2385    0 

table(dpp$RX_ANTIDEPRESSANT,useNA="always")
#   0    1 <NA> 
# 3072  162    0 

dpp$gestdiab <- ifelse(dpp$female==1,dpp$gdm,0)
table(dpp$gestdiab,useNA="always")
#   0    1 <NA> 
# 2880  353    1 

summary(dpp$bmi)
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#  22.13   28.97   32.79   33.97   37.45   70.87 

summary(dpp$aspirindpp)
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max.    NA's 
#  1.000   1.000   1.500   2.066   2.600   6.000       1 

summary(dpp$aspirinos)
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max.    NA's 
#  1.000   1.273   2.636   3.027   4.659   6.000     476 
#  Missings were generally censored earlier

#create combined aspirin variable
dpp$aspirin <- ifelse(!is.na(dpp$aspirindpp)&!is.na(dpp$aspirinos),(dpp$aspirindpp+dpp$aspirinos)/2,
                      ifelse(is.na(dpp$aspirinos),dpp$aspirindpp,dpp$aspirinos))

summary(dpp$aspirin)

summary(dpp$adiponectin)
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max.    NA's 
#  1.800   5.500   7.300   7.933   9.600  35.000     117 

summary(dpp$G000)
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#   76.0   100.0   105.0   106.5   112.0   139.0 

summary(dpp$G120)
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#  140.0   150.0   162.0   164.6   178.0   199.0 

summary(dpp$INV_I000)
#    Min.  1st Qu.   Median     Mean  3rd Qu.     Max.     NA's 
#0.005435 0.030300 0.041670 0.050530 0.062500 0.416700        4 

summary(dpp$HBA1)
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max.    NA's 
#  3.200   5.600   5.900   5.912   6.200   8.500       8 

summary(dpp$cir)
#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max.    NA's 
#0.02158 0.34830 0.54170 0.63430 0.79590 4.64100      60 


dpp$known_cov1 <- ifelse(!is.na(dpp$famhxdb)&!is.na(dpp$HYPERTENSION_HIST)&!is.na(dpp$gestdiab),1,0)    
table(dpp$known_cov1)
# known cov 3048, unknown cov 186


dpp$known_cov2 <- ifelse(!is.na(dpp$famhxdb)&!is.na(dpp$HYPERTENSION_HIST)&!is.na(dpp$gestdiab)&
                    !is.na(dpp$adiponectin)&!is.na(dpp$INV_I000)&!is.na(dpp$HBA1)&!is.na(dpp$cir),1,0)    
table(dpp$known_cov2)
# known cov 2872, unknown cov 362

analy0 <- subset(dpp,known_cov1==1)
analy1 <- subset(analy0,known_cov2==1)
analy2 <- subset(analy1,is.na(analy1$grs)==0)

diab1 <- coxph(Surv(DIABV,DIABF) ~ agerand+lifestyle_tx+drug_tx+female+
famhxdb+HYPERTENSION_HIST+HRT_HIST+DYSLIPID_HIST+
RX_ANTIDEPRESSANT+gestdiab+bmi+aspirin,data=analy0)

diab1.5 <- coxph(Surv(DIABV,DIABF) ~ agerand+lifestyle_tx+drug_tx+female+
famhxdb+HYPERTENSION_HIST+HRT_HIST+DYSLIPID_HIST+
RX_ANTIDEPRESSANT+gestdiab+bmi+aspirin+G000+G120+HBA1,data=analy1)

diab2 <- coxph(Surv(DIABV,DIABF) ~ agerand+lifestyle_tx+drug_tx+female+
famhxdb+HYPERTENSION_HIST+HRT_HIST+DYSLIPID_HIST+
RX_ANTIDEPRESSANT+gestdiab+bmi+aspirin+adiponectin+G000+G120+
INV_I000+HBA1+cir,data=analy1)

ngr1 <- coxph(Surv(NGR_TIME,NGR_EVENT) ~ agerand+lifestyle_tx+drug_tx+female+
famhxdb+HYPERTENSION_HIST+HRT_HIST+DYSLIPID_HIST+
RX_ANTIDEPRESSANT+gestdiab+bmi+aspirin,data=analy0)

ngr1.5 <- coxph(Surv(NGR_TIME,NGR_EVENT) ~ agerand+lifestyle_tx+drug_tx+female+
famhxdb+HYPERTENSION_HIST+HRT_HIST+DYSLIPID_HIST+
RX_ANTIDEPRESSANT+gestdiab+bmi+aspirin+G000+G120+HBA1,data=analy1)

ngr2 <- coxph(Surv(NGR_TIME,NGR_EVENT) ~ agerand+lifestyle_tx+drug_tx+female+
famhxdb+HYPERTENSION_HIST+HRT_HIST+DYSLIPID_HIST+
RX_ANTIDEPRESSANT+gestdiab+bmi+aspirin+adiponectin+G000+G120+
INV_I000+HBA1+cir,data=analy1)

d1c <- 1-analy1$DIABF
kmm1 <- survfit(Surv(analy1$DIABV,d1c)~1)
tau <- 6

MSC_C(1,0.56,analy1$DIABF[analy1$lifestyle_tx==0 & analy1$drug_tx==0],analy1$DIABV[analy1$lifestyle_tx==0 & analy1$drug_tx==0],analy1$NGR_EVENT[analy1$lifestyle_tx==0 & analy1$drug_tx==0],analy1$NGR_TIME[analy1$lifestyle_tx==0 & analy1$drug_tx==0],diab1.5$linear.predictors[analy1$lifestyle_tx==0 & analy1$drug_tx==0],ngr1.5$linear.predictors[analy1$lifestyle_tx==0 & analy1$drug_tx==0],diab2$linear.predictors[analy1$lifestyle_tx==0 & analy1$drug_tx==0],ngr2$linear.predictors[analy1$lifestyle_tx==0 & analy1$drug_tx==0],tau)