rm(list=ls(all=TRUE))
library("coxph.risk")
load("~/Desktop/Lung cancer/lrisk/prog/rpackage/lcmodels/data/best_AIC_models.RData")
risk.bach.10 <- function(data){     
  bach.formula <- function(outcome){
    
    # age = continuous
    # female = binary indicator
    # cpd = cigarettes per day
    # smkyears = years smoked
    # qtyears = years quit
    # asb = asbestos exposure binary indicator
    
    #Formula from appendix of Bach's paper minus coefficients
    formula.string <- "female+
    asb+
    age+
    I(I((age-53.459001)^3)*I(age>53))+
    I(I((age-61.954825)^3)*I(age>61))+
    I(I((age-70.910335)^3)*I(age>70))+
    qtyears+
    I((qtyears)^3)+
    I(I((qtyears-0.50513347)^3)*I(qtyears>0))+
    I(I((qtyears-12.295688)^3)*I(qtyears>12))+
    smkyears+
    I(I((smkyears-27.6577)^3)*I(smkyears>27))+
    I(I((smkyears-40)^3)*I(smkyears>40))+
    I(I((smkyears-50.910335)^3)*I(smkyears>50))+
    cpd+
    I(I((cpd-15)^3)*I(cpd>15))+
    I(I((cpd-20.185718)^3)*I(cpd>20))+
    I(I((cpd-40)^3)*I(cpd>40))"
    
    formula(paste(outcome,"~",formula.string,collapse=""))
  }
  # Coefficients from diagnosis of lung cancer appendix
  bach.coef <- c(intercept =-9.7960571,
                 female =-0.05827261,
                 asb =0.2153936,
                 age=0.070322812,
                 age2=-0.00009382122,
                 age3=0.00018282661,
                 age4=-0.000089005389,
                 qtyears=-0.085684793,
                 qtyears2=0.0065499693,
                 qtyears3=-0.0068305845,
                 qtyears4=0.00028061519,
                 smkyears=0.11425297,
                 smkyears1=-0.000080091477,
                 smkyears2=0.00017069483,
                 smkyears3=-0.000090603358,
                 cpd =0.060818386,
                 cpd1 =-0.00014652216,
                 cpd2 =0.00018486938,
                 cpd3 =-0.000038347226)
  
  # Coefficients from probability of death from something other than lung cancer
  bach.coef.mort <- c(Intercept=-7.2036219, female=-0.49042298, asb=0.06084611, 
                      age=0.099168033, age1=6.2174577e-06, age2=-1.2115774e-05, age3=5.8983164e-06, 
                      qtyears=-0.023358962, qtyears2=0.0019208669, qtyears3=-0.0020031611, qtyears4=8.2294194e-05, 
                      smkyears=0.020041889, smkyears1=6.5443781e-06, smkyears2=-1.3947696e-05, smkyears3=7.4033175e-06, 
                      cpd=0.015490665, cpd1=-1.737645e-05, cpd2=2.1924149e-05, cpd3=-4.5476985e-06)
  
  vars <- c("female",
            "age",
            "cpd",
            "smkyears",
            "qtyears",
            "asb")
  
  var.not.in <- names(data)[!(names(data)%in%vars)][1] # SELECT ARBITRARY VAR FOR OUTCOME
  
  P.lung <- matrix(0, nrow(data), ncol=10)
  P.death <- matrix(0, nrow(data), ncol=10)
  
  for(i in 1:10){
    # Calculate RRs and probabilities
    rr.lung <- model.matrix(bach.formula(var.not.in), data)%*%bach.coef
    rr.death <- model.matrix(bach.formula(var.not.in), data)%*%bach.coef.mort
    P.lung[,i] <- .99629^exp(rr.lung)
    P.death[,i] <- 0.9917663^exp(rr.death)
    # Update age, smoking and quit years for next cycle
    data$age <- data$age+1
    data$smkyears <- data$smkyears+ifelse(data$qtyears==0,1,0) # Add smoking year for current smokers
    data$qtyears <- data$qtyears+ifelse(data$qtyears==0,0,1) # Add quit year for former smokers
  }
  
  # CUMULATIVE PROBABILITIES OF NO CANCER AND NO DEATH
  
  C.lung <- P.lung
  # Delete last column, put 1s in first column and shift other columns over by one
  C.lung[,2:10] <- C.lung[,1:9]
  C.lung[,1] <- 1
  C.lung <- t(apply(C.lung, 1, cumprod))
  
  C.mort <- P.death
  C.mort[,2:10] <- C.mort[,1:9]
  C.mort[,1] <- 1
  C.mort <- t(apply(C.mort, 1, cumprod))
  
  #The final risk calculation
  rowSums((1-P.lung)*P.death*C.lung*C.mort)
} 

risk.tammemagi <- function(data){
  tammemagi.formula <- function(outcome){   
    # age = continuous
    # race = categorical
    # edu = ordinal
    # bmi = continuous
    # fam.lung.cancer = binary
    # prior.cancer = binary
    # copd = binary
    # cpd = cigarettes per day
    # current = binary
    # smkyears = years smoked
    # qtyears = years quit
    formula.string <- "I(age-62)+
    black+hispanic+asian+indian+islander+
    I(edu6-4)+
    I(bmi-27)+
    fam.lung.cancer+
    prior.cancer+
    copd+
    current+
    I(10/cpd-.402154)+
    I(qtyears-10)+
    I(smkyears-27)"              
    
    formula(paste(outcome,"~",formula.string,collapse=""))
  }
  
  tammemagi.coef <- c(intercept=-4.532506,
                      age = 0.0778868,
                      black = 0.3944778,
                      hispanic = -0.7434744,
                      asian = -0.466585,
                      indian = 1.027152,
                      islander = 0,
                      edu = -0.0812744,
                      bmi = -0.0274194,
                      fam.lung.cancer = 0.587185,
                      prior.cancer = 0.4589971,
                      copd = 0.3553063,
                      current = 0.2597431,
                      cpd = -1.822606,
                      qtyears = -0.0308572,
                      smkyears = 0.0317321)
  
  data$black <- 1*(data$race==1)
  data$hispanic <- 1*(data$race==2)
  data$fam.lung.cancer <- 1*(data$fam.lung.trend>0)
  data$current <- 1*(data$qtyears==0)
  vars <- c("age",
            "black","hispanic","asian","islander","indian",
            "edu6",
            "bmi",
            "fam.lung.cancer",
            "prior.cancer",
            "copd",
            "cpd",
            "current",
            "smkyears",
            "qtyears")
  
  var.not.in <- names(data)[!(names(data)%in%vars)][1] # SELECT ARBITRARY VAR FOR OUTCOME
  
  X <- model.matrix(tammemagi.formula(var.not.in), data)
  lp <- X%*%tammemagi.coef
  
  exp(lp)/(1+exp(lp))
} 



risk.kovalchik <- function (begin, end, newdata, coxph1, ...) 
{
  c.coxph.risk <- function (coxph, ...) 
  {
    models <- list(coxph)
    if (!missing(..1)) 
      models <- c(models, list(...))
    models
  }  
  
  coxph.relrisk.uncentered <- function (coxph.object, newdata) 
  {
    center <- coxph.object$means %*% coef(coxph.object)
    if (!missing(newdata)) 
      lp <- predict(coxph.object, newdata, type = "lp")
    else lp <- coxph.object$linear.predictor
    exp(lp + rep(center,length(lp)))
  }  
  
  projection.relrisk <- function (object, data) 
  {
    if (is.numeric(object)) 
      return(object)
    else if (class(object) == "coxph") 
      if (missing(data)) 
        return(coxph.relrisk.uncentered(object))
    else return(coxph.relrisk.uncentered(object, data))
    else stop(cat("No method for class", class(object)))
  }    
  
  #calculate absolute risk given hazards/survival and relative risks
  risk.fixed.interval <- function(H, RR) {
    if (!is.list(H)) 
      0
    else {
      absrisk <- H[[1]]$surv^RR[1] * H[[1]]$haz * RR[1]
      for (i in 2:length(H)) {
        absrisk <- absrisk * (H[[i]]$surv^RR[i])
      }
      sum(absrisk)
    }
  }
  
  models <- c.coxph.risk(coxph1, ...)
  
  #check for missing
  which.kept <- complete.cases(newdata)
  if (!all(which.kept)) {
    warning("Missing cases excluded.")
    if (length(begin) > 1) {
      begin <- begin[which.kept]
      end <- end[which.kept]
    }
    newdata <- newdata[which.kept, ]
  }
  
  #calculate relative risk for each subject and store as list
  rr <- sapply(models, projection.relrisk, data = newdata)
  
  if (is.matrix(rr)) { rr.list <- lapply(1:nrow(rr), function(x) rr[x, ])
  } else { rr.list <- list(rr)}
  
  #estimate risk
  if (length(begin) == 1) {    
    AllVars <- unique(unlist(sapply(models, function(x) all.vars(x$formula))))
    in.interval <- function(x, begin, end) x >= begin & x <= end
    if ("lung.cancer.death" %in% AllVars){
      H <- list(models[[1]]$basehaz[in.interval(models[[1]]$basehaz$time, begin, end),], models[[2]]$basehaz_LCDRAT[in.interval(models[[1]]$basehaz$time, begin, end),])
    } else if ("case" %in% AllVars){
      H <- list(models[[1]]$basehaz[in.interval(models[[1]]$basehaz$time, begin, end),], models[[2]]$basehaz_LCRAT[in.interval(models[[1]]$basehaz$time, begin, end),])        
    }
    #calculate absolute risk given hazards/survival for average covariate values and relative risks
    risks <- mapply(risk.fixed.interval, RR = rr.list, MoreArgs = list(H = H))
  } else {
    risks <- mapply(risk, begin = begin, end = end, RR = rr.list, MoreArgs = list(models = models))
  }
  risks
}

#' Wrapper to the cut() function
#'
#' A wrapper to the cut() function, so that you can automatically break into quantiles as
#' the default behavior, otherwise if the breakpoints are included, then just break on those.
#' In all cases, include.lowest is set to True      
#' @export
cuts <- function(data,npieces,simple.labels=T,...) {
  
  if (length(npieces)==0 | any(is.na(npieces)) | any(!is.numeric(npieces)))
    stop("npieces must be a numeric scalar or a vector of breakpoints")
  
  if (length(npieces)==1)
    # Just break into quantiles.  Use quantile labelling: Q1,Q2,Q3,etc. if you want
    if (simple.labels)
      cutdata <- cut(data,breaks=quantile(data,seq(0,1,1/npieces),na.rm=T,...),
                     include.lowest=T,labels=paste("Q",1:npieces,sep=""),...)
    else
      cutdata <- cut(data,breaks=quantile(data,seq(0,1,1/npieces),na.rm=T,...),
                     include.lowest=T,...)
    else
      # Break into pieces as specified by the breakpoints
      if (simple.labels)
        cutdata <- cut(data,breaks=npieces,include.lowest=T,
                       labels=paste("Q",1:(length(npieces)-1),sep=""),...)
      else
        cutdata <- cut(data,breaks=npieces,include.lowest=T,...)
      
      return(cutdata)
}

load(file="~/Desktop/Lung cancer/lrisk/other/nlst/anlst_010918.RData")
source("~/Desktop/dissertation/aoas submission/simulation studies/MSC_C_simple.R")
source("~/Desktop/dissertation/aoas submission/simulation studies/MSC_C_3.R")

anlst$incidence.years <- anlst$incidence.days/365.25
anlst <- subset(anlst,screen_group=="X-ray")  #26554

persons <- cbind(anlst$age,
                 anlst$female,
                 anlst$smkyears,
                 anlst$qtyears,
                 anlst$cpd,
                 as.numeric(as.character(anlst$race)),
                 anlst$emp,
                 anlst$fam.lung.trend,
                 anlst$bmi,
                 anlst$edu6,
                 anlst$asb,
                 anlst$pneu,
                 anlst$prior.cancer,
                 as.numeric(as.character(anlst$fam.cancer.onset)),
                 anlst$dust,
                 anlst$fam.cancer,
                 anlst$fam.smoke.cancer,
                 anlst$no.asthma,
                 anlst$asian,
                 anlst$islander,
                 anlst$indian)

x <- as.matrix(persons)

age <- x[,1]
female <- x[,2]
smkyears <- x[,3]
qtyears <- x[,4]
qtyears <- ifelse(is.na(qtyears),0,qtyears)
cpd <- x[,5]
race <- as.factor(x[,6])
emp <- x[,7]
fam.lung.trend <- x[,8]
bmi <- x[,9]
edu6 <- x[,10]
pkyr.cat <- smkyears*cpd/20
age.stopped <- age-qtyears
smoke.age.start <- age-smkyears-qtyears
asb <- x[,11]
pneu <- x[,12]
prior.cancer <- x[,13]
fam.cancer.onset <- as.factor(x[,14])
copd <- emp
dust <- x[,15]
fam.cancer <- x[,16]
fam.smoke.cancer <- x[,17]
no.hayfever <- x[,18]
asian <- x[,19]
islander <- x[,20]
indian <- x[,21]

covar <- data.frame(age=age,
                    bmi=bmi,
                    cpd=cpd,
                    emp=emp,
                    fam.lung.trend=fam.lung.trend,
                    female=female,
                    qtyears=qtyears,
                    smkyears=smkyears,
                    race=race,
                    edu6=edu6,
                    pkyr.cat=pkyr.cat,
                    asb=asb,
                    smoke.age.start=smoke.age.start,
                    pneu = pneu,
                    prior.cancer = prior.cancer,
                    fam.cancer.onset = fam.cancer.onset,
                    copd=copd,
                    dust=dust,
                    fam.cancer=fam.cancer,
                    fam.smoke.cancer=fam.smoke.cancer,
                    no.hayfever=no.hayfever,
                    age.stopped=age.stopped,
                    asian = asian,
                    islander = islander,
                    indian = indian)

### Bach 2003 ###   
anlst$bach <- risk.bach.10(covar)
anlst$plcom2012 <- risk.tammemagi(covar)
anlst$LCDRAT <- risk.kovalchik(0, 5, covar[,1:11], LCDRAT, cox.death)
anlst$LCRAT <- risk.kovalchik(0, 5, covar[,1:11], LCRAT, cox.death)

#if follow-up on incidence exists beyond last mortality linkage, update with alive as of last incidence follow-up
anlst$years.followed[which(anlst$years.followed<anlst$incidence.years)] <- anlst$incidence.years[which(anlst$years.followed<anlst$incidence.years)]
#1158 blacks, 456 hispanics, 524 asian
#deaths: 29 blacks, 4 hispanics (1 hispanic not previously diagnosed), 8 asian
#incidence: 51 blacks, 5 hispanic, 17 asian

#individuals who died of lung cancer that they were not previously diagnosed for lung cancer - have to decide what to do with these
anlst$event.lung.cancer[which(anlst$years.followed==anlst$incidence.years & anlst$lung.cancer.death==1)]

black <- subset(anlst,black==1)
hispanic <- subset(anlst,hispanic==1)
asian <- subset(anlst,asian==1)

tau <- 6
d1c <- 1-black$lung.cancer.death
kmm1 <- survfit(Surv(black$years.followed,d1c)~ 1)
min(kmm1$surv[kmm1$time<=tau]) #0.0773085

d1c <- 1-hispanic$lung.cancer.death
kmm1 <- survfit(Surv(hispanic$years.followed,d1c)~ 1)
min(kmm1$surv[kmm1$time<=tau]) #0.1019474

d1c <- 1-asian$lung.cancer.death
kmm1 <- survfit(Surv(asian$years.followed,d1c)~ 1)
min(kmm1$surv[kmm1$time<=tau]) #0.117119

black.bach <- MSC_Cs(1,1,black$lung.cancer.death,black$years.followed,black$event.lung.cancer,black$incidence.years,black$bach,black$bach,tau)
black.lcdrat <- MSC_Cs(1,1,black$lung.cancer.death,black$years.followed,black$event.lung.cancer,black$incidence.years,black$LCDRAT,black$LCDRAT,tau)
black.lcrat <-  MSC_Cs(1,1,black$lung.cancer.death,black$years.followed,black$event.lung.cancer,black$incidence.years,black$LCRAT,black$LCRAT,tau)
black.m2012 <- MSC_Cs(1,1,black$lung.cancer.death,black$years.followed,black$event.lung.cancer,black$incidence.years,black$plcom2012,black$plcom2012,tau)

black.comp <- MSC_C(1,1,black$lung.cancer.death,black$years.followed,black$event.lung.cancer,black$incidence.years,black$bach,black$bach,black$LCDRAT,black$LCDRAT,tau)

hispanic.bach <- MSC_Cs(1,1,hispanic$lung.cancer.death,hispanic$years.followed,hispanic$event.lung.cancer,hispanic$incidence.years,hispanic$bach,hispanic$bach,tau)
hispanic.lcdrat <- MSC_Cs(1,1,hispanic$lung.cancer.death,hispanic$years.followed,hispanic$event.lung.cancer,hispanic$incidence.years,hispanic$LCDRAT,hispanic$LCDRAT,tau)
hispanic.lcrat <- MSC_Cs(1,1,hispanic$lung.cancer.death,hispanic$years.followed,hispanic$event.lung.cancer,hispanic$incidence.years,hispanic$LCRAT,hispanic$LCRAT,tau)
hispanic.m2012 <- MSC_Cs(1,1,hispanic$lung.cancer.death,hispanic$years.followed,hispanic$event.lung.cancer,hispanic$incidence.years,hispanic$plcom2012,hispanic$plcom2012,tau)

tau <- 6
asian.bach2 <- MSC_Cs(1,1,asian$lung.cancer.death,asian$years.followed,asian$event.lung.cancer,asian$incidence.years,asian$bach,asian$bach,tau)
asian.lcdrat2 <- MSC_Cs(1,1,asian$lung.cancer.death,asian$years.followed,asian$event.lung.cancer,asian$incidence.years,asian$LCDRAT,asian$LCDRAT,tau)
asian.lcrat2 <- MSC_Cs(1,1,asian$lung.cancer.death,asian$years.followed,asian$event.lung.cancer,asian$incidence.years,asian$LCRAT,asian$LCRAT,tau)
asian.m20122 <- MSC_Cs(1,1,asian$lung.cancer.death,asian$years.followed,asian$event.lung.cancer,asian$incidence.years,asian$plcom2012,asian$plcom2012,tau)

save(black.bach,black.lcdrat,black.lcrat,black.m2012,
     hispanic.bach,hispanic.lcdrat,hispanic.lcrat,hispanic.m2012,
     asian.bach,asian.lcdrat,asian.lcrat,asian.m2012,
     file="lcrisks_compare.RData")