#Table 1: false discovery rate simulations
#illness-death
#predictive of death only

#### Clear Workspace + Load Packages
rm(list=ls(all=TRUE))
library(survival)
library("mvtnorm")
set.seed(1234)
dirname <- "/home/cheunglc/cindex2"

#### number of sims to use
nsim <- 100
#### number of observations to use
n <- 15000
num <- n*nsim

#### set covariate
#true parameter values
beta01 <- 2.9
beta02 <- 3.7
beta1 <- -1 #lower is riskier
beta2 <- -.5
tau <- 1


#step 1: draw covariate to use for each simulation
sigma <- matrix(c( 3,.5,.5,.5,.5,.5, 0, 0, 0, 0, 0,
                   .5, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                   .5, 0, 3, 0, 0, 0, 0, 0, 0, 0, 0,
                   .5, 0, 0, 3, 0, 0, 0, 0, 0, 0, 0,
                   .5, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0,
                   .5, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0,
                   0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0,
                   0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0,
                   0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0,
                   0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0,
                   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3), ncol=11)
x <- rmvnorm(num,sigma=sigma)
x1 <- rnorm(num,0,sqrt(3))
z1 <- beta1*(x1)
z2 <- beta2*(x[,1])
lp1 <- exp(beta01+z1)
lp2 <- exp(beta02+z2)

t_tilde2 <- rweibull(num,1/tau,lp1) #time from healthy to illness
t_tilde1 <- rweibull(num,1/tau,lp2) #time from illness to death

#time to death
t_tilde10 <- t_tilde1+t_tilde2

#### set censoring
cens <- pmin(rexp(nsim*n,.05),runif(nsim*n,24,30))  #random censoring

#cens <- rep(13.5,num)  #type I censoring
t1 <- pmin(t_tilde10,cens)
t2 <- pmin(t_tilde2,cens,t_tilde10)
d1 <- I(t_tilde10<=cens)
d2 <- I(t_tilde2<=cens)  #indicator variable for illness
xsam_all <- data.frame(t1,d1,t2,d2,x)

#### percent of event 1 observed
table(d1)/length(d1)  #0.855

#### percent of event 2 observed
table(d2)/length(d2)  #0.505

d1c <- 1-d1
kmm1 <- survfit(Surv(t1,d1c)~ 1)
tau1 <- max(kmm1$time[kmm1$surv>=.25])

################
#true estimates#
################
d1tau <- t_tilde10 <= tau1
t_tilde10_tau <- ifelse(t_tilde10<=tau1,t_tilde10,tau1)
coxmod1 <- coxph(Surv(t_tilde10_tau,d1tau) ~ xsam_all$X1)   ### fit cox model to diabetes
summary(coxmod1)$concordance  #0.6842355856

d1tau <- t_tilde10 <= tau1
t_tilde10_tau <- ifelse(t_tilde10<=tau1,t_tilde10,tau1)
coxmod1 <- coxph(Surv(t_tilde10_tau,d1tau) ~ xsam_all$X2)   ### fit cox model to illness
summary(coxmod1)$concordance  #0.5296562540

d2tau <- t_tilde2 <= tau1
t_tilde2_tau <- ifelse(t_tilde2<=tau1,t_tilde2,tau1)

data <- data.frame(t_tilde10,d1tau,t_tilde2,d2tau,x)
res <- data.frame()
for (k in 1:100){
dat <- data[((k-1)*n+1):(k*n),]
neworder <- order(dat$t_tilde10,dat$d1tau)    
dat <- dat[neworder,]
d <- dat$d1tau
t <- dat$t_tilde10
z1 <- dat$X1
z2 <- dat$X2
txx1_ij <- matrix(0,n,n)  #for C - usable
txy1_ij1 <- matrix(0,n,n) #for C - Z1
txy1_ij2 <- matrix(0,n,n) #for C - Z2
caseorder <- which(d==1)
for (i in 1:length(caseorder)){
  if (caseorder[i]!=n) {
    txx1_ij[caseorder[i],(caseorder[i]+1):n] <- rep(1,(n-caseorder[i]))
    txx1_ij[(caseorder[i]+1):n,caseorder[i]] <- rep(1,(n-caseorder[i]))     
    txy1_ij1[caseorder[i],(caseorder[i]+1):n] <- I(z1[(caseorder[i]+1):n]<=z1[caseorder[i]]) - I(z1[(caseorder[i]+1):n]>=z1[caseorder[i]])
    txy1_ij1[(caseorder[i]+1):n,caseorder[i]] <- I(z1[(caseorder[i]+1):n]<=z1[caseorder[i]]) - I(z1[(caseorder[i]+1):n]>=z1[caseorder[i]])
    txy1_ij2[caseorder[i],(caseorder[i]+1):n] <- I(z2[(caseorder[i]+1):n]<=z2[caseorder[i]]) - I(z2[(caseorder[i]+1):n]>=z2[caseorder[i]])
    txy1_ij2[(caseorder[i]+1):n,caseorder[i]] <- I(z2[(caseorder[i]+1):n]<=z2[caseorder[i]]) - I(z2[(caseorder[i]+1):n]>=z2[caseorder[i]])    
  }
}

newdata <- subset(dat,d1tau==0)
neworder2 <- order(newdata$t_tilde2,newdata$d2tau)
newdata <- newdata[neworder2,]
d <- newdata$d2tau
t <- newdata$t_tilde2
z1 <- newdata$X1
z2 <- newdata$X2
num2 <- length(d) 

txx2_ij <- matrix(0,num2,num2)  #for C - usable
txy2_ij1 <- matrix(0,num2,num2) #for C - Z1
txy2_ij2 <- matrix(0,num2,num2) #for C - Z2
caseorder <- which(d==1)
for (i in 1:length(caseorder)){
  if (caseorder[i]!=num2) {
    txx2_ij[caseorder[i],(caseorder[i]+1):num2] <- rep(1,(num2-caseorder[i]))
    txx2_ij[(caseorder[i]+1):num2,caseorder[i]] <- rep(1,(num2-caseorder[i]))     
    txy2_ij1[caseorder[i],(caseorder[i]+1):num2] <- I(z1[(caseorder[i]+1):num2]<=z1[caseorder[i]]) - I(z1[(caseorder[i]+1):num2]>=z1[caseorder[i]])
    txy2_ij1[(caseorder[i]+1):num2,caseorder[i]] <- I(z1[(caseorder[i]+1):num2]<=z1[caseorder[i]]) - I(z1[(caseorder[i]+1):num2]>=z1[caseorder[i]])
    txy2_ij2[caseorder[i],(caseorder[i]+1):num2] <- I(z2[(caseorder[i]+1):num2]<=z2[caseorder[i]]) - I(z2[(caseorder[i]+1):num2]>=z2[caseorder[i]])
    txy2_ij2[(caseorder[i]+1):num2,caseorder[i]] <- I(z2[(caseorder[i]+1):num2]<=z2[caseorder[i]]) - I(z2[(caseorder[i]+1):num2]>=z2[caseorder[i]])
  }
}
denom <- sum(txx1_ij)+.75*sum(txx2_ij)
numer1 <- sum(txy1_ij1)+.75*sum(txy2_ij1)
cstar1 <- .5*(numer1/denom + 1)  
numer2 <- sum(txy1_ij2)+.75*sum(txy2_ij2)
cstar2 <- .5*(numer2/denom + 1)  
res <- rbind(res,c(cstar1,cstar2))
}
colMeans(res)
#0.573032           
#0.511690
